/******************************************************************************
 * MODULE       : AmbientLightSensor.cpp
 * FUNCTION     : Hardware Abstraction Layer of Sensor service in Android Framework
 *              : This is header file of light sensor
 * AUTHOR       : Masafumi Seike <masafumi.seike@dsn.rohm.co.jp >
 * PROGRAMMED   : Sensor Application Development Group, ROHM CO.,LTD.
 * MODIFICATION : Modified by ROHM, JUN/11/2012
 * REMARKS      :
 * COPYRIGHT    : Copyright (C) 2008 The Android Open Source Project
 *              : Copyright (C) 2011 - 2012 ROHM CO.,LTD.
 *              : Licensed under the Apache License, Version 2.0 (the "License");
 *              : you may not use this file except in compliance with the License.
 *              : You may obtain a copy of the License at
 *              :
 *              :      http://www.apache.org/licenses/LICENSE-2.0
 *              :
 *              : Unless required by applicable law or agreed to in writing, software
 *              : distributed under the License is distributed on an "AS IS" BASIS,
 *              : WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *              : See the License for the specific language governing permissions and
 *              : limitations under the License.
 *****************************************************************************/

/*****************************************************************************
 * Include                                                                   *
 *****************************************************************************/
#include <errno.h>
#include <linux/input.h>
#include <cutils/log.h>
#include "sensors.h"
#include "AmbientLightSensor.h"
#include "rohm_bh1730_i2c_if.h"

/******************************************************************************
 * NAME       : AmbientLightSensor::AmbientLightSensor
 * FUNCTION   : initialize the parameter
 * REMARKS    :
 *****************************************************************************/
AmbientLightSensor::AmbientLightSensor()
    : SensorBase(ROHM_DEV_BH1730, ROHM_INPUT_BH1730),
      mEnabled(0),
      mEventsSinceEnable(0),
      mInputReader(4),
      mHasPendingEvent(false)
{
    mPendingEvent.version = sizeof(sensors_event_t);
    mPendingEvent.sensor  = SENSORS_LIGHT_HANDLE;
    mPendingEvent.type    = SENSOR_TYPE_LIGHT;
    memset(mPendingEvent.data, 0, sizeof(mPendingEvent.data));

    if (dev_fd) {
        enable(0, 0);
    }
}

/******************************************************************************
 * NAME       : AmbientLightSensor::~AmbientLightSensor
 * FUNCTION   : stop sensor device
 * REMARKS    :
 *****************************************************************************/
AmbientLightSensor::~AmbientLightSensor() {
    if (mEnabled) {
        enable(0, 0);
    }
}

/******************************************************************************
 * NAME       : AmbientLightSensor::setDelay
 * FUNCTION   : set delay time to sensor device
 * REMARKS    :
 *****************************************************************************/
int AmbientLightSensor::setDelay(int32_t handle, int64_t ns)
{
#define TIME_OFFSET (125)

    int result;
    int timer_delay;

    /* set the given value from application */
    timer_delay = (ns / 1000000) + TIME_OFFSET;
    result      = ioctl(dev_fd, IOCTL_APP_SET_TIMER, &timer_delay);
    if (result < 0) {
        LOGE("SET_TIMER of ioctl is issue NG by device !\n");
        LOGE("error value = 0x%x !\n", result);
        return (result);
    }

    return (0);

#undef TIME_OFFSET
}

/******************************************************************************
 * NAME       : AmbientLightSensor::enable
 * FUNCTION   : stop or start sensor device
 * REMARKS    :
 *****************************************************************************/
int AmbientLightSensor::enable(int32_t handle, int en)
{
    POWERON_ALS_ARG pwr_als;
    int             flags = en ? 1 : 0;
    int             result;

    mEventsSinceEnable = 0;
    mPreviousLight     = -1;
    if (flags != mEnabled) {
        if (0 == flags) {
            pwr_als.power_als = ALS_DISABLE;
        } else {
            pwr_als.power_als = ALS_ENABLE;
        }
        pwr_als.intr = ALS_DISABLE;
        result = ioctl(dev_fd, IOCTL_APP_SET_PWRSET_ALS, &pwr_als);
        if (result < 0) {
            LOGE("device ioctl NG !\n");
            LOGE("error value = 0x%x !\n", result);
            return (result);
        }
        mEnabled = flags;
    }
    return (0);
}

/******************************************************************************
 * NAME       : AmbientLightSensor::hasPendingEvents
 * FUNCTION   : return the staying event
 * REMARKS    :
 *****************************************************************************/
bool AmbientLightSensor::hasPendingEvents() const {
    return (mHasPendingEvent);
}

/******************************************************************************
 * NAME       : AmbientLightSensor::readEvents
 * FUNCTION   : handle sensor event
 * REMARKS    :
 *****************************************************************************/
int AmbientLightSensor::readEvents(sensors_event_t *data, int count)
{
#define FIRST_GOOD_EVENT (1)
#define CUT_UNIT         (1000)

    float calc_data;
    long  cut_data;

    if (count < 1) {
        return (-EINVAL);
    }

    if (mHasPendingEvent) {
        mHasPendingEvent        = false;
        mPendingEvent.timestamp = getTimestamp();
        *data                   = mPendingEvent;
        return (mEnabled ? 1 : 0);
    }

    ssize_t n = mInputReader.fill(data_fd);
    if (n < 0) {
        return (n);
    }

    int               numEventReceived = 0;
    input_event const *event;

    while (count && mInputReader.readEvent(&event)) {
        if (event->type == EV_ABS) {
            switch (event->code) {
            case ABS_X:
                calc_data           = (float)event->value / (1 << DECIMAL_BIT);
                cut_data            = (long)(calc_data * CUT_UNIT);
                mPendingEvent.light = (float)cut_data / CUT_UNIT;
                if (mEventsSinceEnable < FIRST_GOOD_EVENT) {
                    mEventsSinceEnable++;
                }
                break;
            default:
                break;
            }
        } else if (event->type == EV_SYN) {
            mPendingEvent.timestamp = timevalToNano(event->time);
            if (mEnabled && (mPendingEvent.light != mPreviousLight) && (mEventsSinceEnable >= FIRST_GOOD_EVENT)) {
                *data++ = mPendingEvent;
                count--;
                numEventReceived++;
                mPreviousLight = mPendingEvent.light;
            }
        } else {
            LOGE("unknown event (type=%d, code=%d)", event->type, event->code);
        }
        mInputReader.next();
    }

    return (numEventReceived);

#undef FIRST_GOOD_EVENT
#undef CUT_UNIT

}
