/******************************************************************************
 * MODULE       : InputEventReader.cpp
 * FUNCTION     : Hardware Abstraction Layer of Sensor service in Android Framework
 * AUTHOR       : Masafumi Seike <masafumi.seike@dsn.rohm.co.jp >
 * PROGRAMMED   : Sensor Application Development Group, ROHM CO.,LTD.
 * MODIFICATION : Modified by ROHM, JUN/11/2012
 * REMARKS      :
 * COPYRIGHT    : Copyright (C) 2008 The Android Open Source Project
 *              : Copyright (C) 2011 - 2012 ROHM CO.,LTD.
 *              : Licensed under the Apache License, Version 2.0 (the "License");
 *              : you may not use this file except in compliance with the License.
 *              : You may obtain a copy of the License at
 *              :
 *              :      http://www.apache.org/licenses/LICENSE-2.0
 *              :
 *              : Unless required by applicable law or agreed to in writing, software
 *              : distributed under the License is distributed on an "AS IS" BASIS,
 *              : WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *              : See the License for the specific language governing permissions and
 *              : limitations under the License.
 *****************************************************************************/

/*****************************************************************************
 * Include                                                                   *
 *****************************************************************************/
#include <linux/input.h>
#include <unistd.h>
#include <errno.h>
#include "InputEventReader.h"

/*****************************************************************************/

/*****************************************************************************
 * Functions                                                                 *
 *****************************************************************************/
/******************************************************************************
 * NAME       : InputEventCircularReader::InputEventCircularReader
 * FUNCTION   : get memory and initialize the parameter
 * REMARKS    :
 *****************************************************************************/
InputEventCircularReader::InputEventCircularReader(size_t numEvents)
    : mBuffer(new input_event[numEvents * 2]),
      mBufferEnd(mBuffer + numEvents),
      mHead(mBuffer),
      mCurr(mBuffer),
      mFreeSpace(numEvents)
{
}

/******************************************************************************
 * NAME       : sensors_poll_context_t::sensors_poll_context_t
 * FUNCTION   : free memory
 * REMARKS    :
 *****************************************************************************/
InputEventCircularReader::~InputEventCircularReader()
{
    if (mBuffer != NULL) {
        delete [] mBuffer;
    }
}

/******************************************************************************
 * NAME       : InputEventCircularReader::fill
 * FUNCTION   : get input event data
 * REMARKS    :
 *****************************************************************************/
ssize_t InputEventCircularReader::fill(int fd)
{
    if (mBuffer == NULL) {
        return (-1);
    }

    size_t numEventsRead = 0;
    if (mFreeSpace) {
        const ssize_t nread = read(fd, mHead, mFreeSpace * sizeof(input_event));
        if ((nread < 0) || (nread % sizeof(input_event))) {
            // we got a partial event!!
            return ((nread < 0) ? -errno : -EINVAL);
        }

        numEventsRead = nread / sizeof(input_event);
        if (numEventsRead) {
            mHead      += numEventsRead;
            mFreeSpace -= numEventsRead;
            if (mHead > mBufferEnd) {
                size_t s = mHead - mBufferEnd;
                memcpy(mBuffer, mBufferEnd, s * sizeof(input_event));
                mHead = mBuffer + s;
            }
        }
    }

    return (numEventsRead);
}

/******************************************************************************
 * NAME       : InputEventCircularReader::readEvent
 * FUNCTION   : confirm input data which is left to buffer
 * REMARKS    :
 *****************************************************************************/
ssize_t InputEventCircularReader::readEvent(input_event const **events)
{
    if (mBuffer == NULL) {
        return (0);
    }

    *events = mCurr;
    ssize_t available = (mBufferEnd - mBuffer) - mFreeSpace;
    return (available ? 1 : 0);
}

/******************************************************************************
 * NAME       : InputEventCircularReader::next
 * FUNCTION   : move to next data
 * REMARKS    :
 *****************************************************************************/
void InputEventCircularReader::next()
{
    mCurr++;
    mFreeSpace++;
    if (mCurr >= mBufferEnd) {
        mCurr = mBuffer;
    }
}
