/******************************************************************************
 * MODULE       : SensorBase.cpp
 * FUNCTION     : Hardware Abstraction Layer of Sensor service in Android Framework
 *              : This is source file of base class
 * AUTHOR       : Masafumi Seike <masafumi.seike@dsn.rohm.co.jp >
 * PROGRAMMED   : Sensor Application Development Group, ROHM CO.,LTD.
 * MODIFICATION : Modified by ROHM, JUN/11/2012
 * REMARKS      :
 * COPYRIGHT    : Copyright (C) 2008 The Android Open Source Project
 *              : Copyright (C) 2011 - 2012 ROHM CO.,LTD.
 *              : Licensed under the Apache License, Version 2.0 (the "License");
 *              : you may not use this file except in compliance with the License.
 *              : You may obtain a copy of the License at
 *              :
 *              :      http://www.apache.org/licenses/LICENSE-2.0
 *              :
 *              : Unless required by applicable law or agreed to in writing, software
 *              : distributed under the License is distributed on an "AS IS" BASIS,
 *              : WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *              : See the License for the specific language governing permissions and
 *              : limitations under the License.
 *****************************************************************************/

/*****************************************************************************
 * Include                                                                   *
 *****************************************************************************/
#include <cutils/log.h>
#include <errno.h>
#include <dirent.h>
#include <linux/input.h>
#include "sensors.h"
#include "SensorBase.h"
/*****************************************************************************/

/*****************************************************************************
 * Functions                                                                 *
 *****************************************************************************/
/******************************************************************************
 * NAME       : SensorBase::SensorBase
 * FUNCTION   : get file descriptor and initialize the parameter
 * REMARKS    :
 *****************************************************************************/
SensorBase::SensorBase(const char *dev_name, const char *data_name)
    : dev_name(dev_name),
      data_name(data_name),
      dev_fd(-1),
      data_fd(-1)
{
    if (data_name) {
        data_fd = openInput(data_name);
    }
    if (dev_name) {
        dev_fd = open_device(dev_name);
    }
}

/******************************************************************************
 * NAME       : SensorBase::~SensorBase
 * FUNCTION   : free file descriptor
 * REMARKS    :
 *****************************************************************************/
SensorBase::~SensorBase()
{
    if (data_fd >= 0) {
        close(data_fd);
    }
    close_device();
}

/******************************************************************************
 * NAME       : SensorBase::open_device
 * FUNCTION   : get file descriptor of I/O device
 * REMARKS    :
 *****************************************************************************/
int SensorBase::open_device(const char *dev_name)
{
    int fd = -1;
    if (dev_fd < 0 && dev_name) {
        fd = open(dev_name, O_RDWR);
        LOGE_IF(fd < 0, "couldn't open %s (%s)", dev_name, strerror(errno));
    }

    return (fd);
}

/******************************************************************************
 * NAME       : SensorBase::close_device
 * FUNCTION   : free file descriptor of I/O device
 * REMARKS    :
 *****************************************************************************/
int SensorBase::close_device()
{
    if (dev_fd >= 0) {
        close(dev_fd);
        dev_fd = -1;
    }

    return (0);
}

/******************************************************************************
 * NAME       : SensorBase::getFd
 * FUNCTION   : get file descriptor of Input device or I/O device
 * REMARKS    :
 *****************************************************************************/
int SensorBase::getFd() const
{
    return (data_fd);
}

/******************************************************************************
 * NAME       : SensorBase::setDelay
 * FUNCTION   : non process
 * REMARKS    :
 *****************************************************************************/
int SensorBase::setDelay(int32_t handle, int64_t ns)
{
    return (0);
}

/******************************************************************************
 * NAME       : SensorBase::setDelay
 * FUNCTION   : return the error number
 * REMARKS    :
 *****************************************************************************/
bool SensorBase::hasPendingEvents() const
{
    return (false);
}

/******************************************************************************
 * NAME       : SensorBase::getTimestamp
 * FUNCTION   : get time stamp
 * REMARKS    :
 *****************************************************************************/
int64_t SensorBase::getTimestamp()
{
    struct timespec t;

    t.tv_sec = t.tv_nsec = 0;
    clock_gettime(CLOCK_MONOTONIC, &t);

    return (int64_t(t.tv_sec) * 1000000000LL + t.tv_nsec);
}

/******************************************************************************
 * NAME       : SensorBase::openInput
 * FUNCTION   : get file descriptor of Input device
 * REMARKS    :
 *****************************************************************************/
int SensorBase::openInput(const char *inputName)
{
#define NAME_SPACE (80)

    int           fd       = -1;
    const char    *dirname = "/dev/input";
    char          devname[PATH_MAX];
    char          *filename;
    DIR           *dir;
    struct dirent *de;

    dir = opendir(dirname);
    if (dir == NULL) {
        return (-1);
    }
    strcpy(devname, dirname);
    filename    = devname + strlen(devname);
    *filename++ = '/';
    while ((de = readdir(dir))) {
        if (de->d_name[0] == '.' && \
            (de->d_name[1] == '\0' || \
             (de->d_name[1] == '.' && de->d_name[2] == '\0'))) {
            continue;
        }
        strcpy(filename, de->d_name);
        fd = open(devname, O_RDONLY);
        if (fd >= 0) {
            char name[NAME_SPACE];
            if (ioctl(fd, EVIOCGNAME(sizeof(name) - 1), &name) < 1) {
                name[0] = '\0';
            }
            if (!strcmp(name, inputName)) {
                strcpy(input_name, filename);
                break;
            } else {
                close(fd);
                fd = -1;
            }
        }
    }
    closedir(dir);
    LOGE_IF(fd < 0, "couldn't find '%s' input device", inputName);

    return (fd);

#undef NAME_SPACE
}
